@@ -1,15 +1,68 @@
+# BEGIN BPS TAGGED BLOCK {{{
+# COPYRIGHT:
+# 
+# This software is Copyright (c) 2003-2006 Best Practical Solutions, LLC
+#                                          <clkao@bestpractical.com>
+# 
+# (Except where explicitly superseded by other copyright notices)
+# 
+# 
+# LICENSE:
+# 
+# 
+# This program is free software; you can redistribute it and/or
+# modify it under the terms of either:
+# 
+#   a) Version 2 of the GNU General Public License.  You should have
+#      received a copy of the GNU General Public License along with this
+#      program.  If not, write to the Free Software Foundation, Inc., 51
+#      Franklin Street, Fifth Floor, Boston, MA 02110-1301 or visit
+#      their web page on the internet at
+#      http://www.gnu.org/copyleft/gpl.html.
+# 
+#   b) Version 1 of Perl's "Artistic License".  You should have received
+#      a copy of the Artistic License with this package, in the file
+#      named "ARTISTIC".  The license is also available at
+#      http://opensource.org/licenses/artistic-license.php.
+# 
+# This work is distributed in the hope that it will be useful, but
+# WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+# General Public License for more details.
+# 
+# CONTRIBUTION SUBMISSION POLICY:
+# 
+# (The following paragraph is not intended to limit the rights granted
+# to you to modify and distribute this software under the terms of the
+# GNU General Public License and is only of importance to you if you
+# choose to contribute your changes and enhancements to the community
+# by submitting them to Best Practical Solutions, LLC.)
+# 
+# By intentionally submitting any modifications, corrections or
+# derivatives to this work, or any other work intended for use with SVK,
+# to Best Practical Solutions, LLC, you confirm that you are the
+# copyright holder for those contributions and you grant Best Practical
+# Solutions, LLC a nonexclusive, worldwide, irrevocable, royalty-free,
+# perpetual, license to use, copy, create derivative works based on
+# those contributions, and sublicense and distribute those contributions
+# and any derivatives thereof.
+# 
+# END BPS TAGGED BLOCK }}}
 package SVK::Command;
 use strict;
+use base qw(App::CLI App::CLI::Command);
 use SVK::Version;  our $VERSION = $SVK::VERSION;
 use Getopt::Long qw(:config no_ignore_case bundling);
 
+use SVK::Logger;
 use SVK::Util qw( get_prompt abs2rel abs_path is_uri catdir bsd_glob from_native
-		  find_svm_source $SEP IS_WIN32 HAS_SVN_MIRROR catdepot traverse_history);
+		  find_svm_source $SEP IS_WIN32 catdepot traverse_history);
 use SVK::I18N;
 use Encode;
+use constant subcommands => '*';
 
 use Class::Autouse
-    qw( SVK::Target SVK::Notify
+    qw( Path::Class SVK::Path SVK::Path::Checkout SVK::Notify
 	SVK::Editor::Status SVK::Editor::Diff
 	Pod::Simple::Text SVK::Merge );
 
@@ -38,7 +91,8 @@ information is displayed instead.
 
 =cut
 
-my %alias = qw( ann		annotate
+use constant alias =>
+            qw( ann		annotate
                 blame		annotate
                 praise		annotate
 		co		checkout
@@ -77,67 +131,17 @@ my %alias = qw( ann		annotate
 		ver		version
 	    );
 
+use constant global_options => ( 'h|help|?'   => 'help',
+				 'encoding=s' => 'encoding',
+				 'ignore=s@'  => 'ignore',
+			       );
+
+my %alias = alias;
 my %cmd2alias = map { $_ => [] } values %alias;
 while( my($alias, $cmd) = each %alias ) {
     push @{$cmd2alias{$cmd}}, $alias;
 }
 
-
-=head3 new ($xd)
-
-Base constructor for all commands.
-
-=cut
-
-sub new {
-    my ($class, $xd) = @_;
-    my $self = bless { xd => $xd }, $class;
-    return $self;
-}
-
-=head3 get_cmd ($cmd, $xd)
-
-Load the command subclass specified in C<$cmd>, and return a new
-instance of it, populated with C<$xd>.  Command aliases are handled here.
-
-To construct a command object from another command object, use the
-C<command> instance method instead.
-
-=cut
-
-sub get_cmd {
-    my ($pkg, $cmd, $xd) = @_;
-    die "Command not recognized, try $0 help.\n"
-	unless $cmd =~ m/^[?a-z]+$/;
-    $pkg = join('::', 'SVK::Command', _cmd_map ($cmd));
-    my $file = "$pkg.pm";
-    $file =~ s!::!/!g;
-
-    unless (eval {require $file; 1} and $pkg->can('run')) {
-	warn $@ if $@ and exists $INC{$file};
-	die "Command not recognized, try $0 help.\n";
-    }
-    $pkg->new ($xd);
-}
-
-sub _cmd_map {
-    my ($cmd) = @_;
-    $cmd = $alias{$cmd} if exists $alias{$cmd};
-    $cmd =~ s/^(.)/\U$1/;
-    return $cmd;
-}
-
-# rebless to subcommand class if it exists
-sub _subcommand {
-    my ($self) = @_;
-    no strict 'refs';
-    for (grep {$self->{$_}} values %{{$self->options}}) {
-	if (exists ${ref($self).'::'}{$_.'::'}) {
-	    return bless ($self, (ref($self)."::$_"));
-	}
-    }
-}
-
 =head3 invoke ($xd, $cmd, $output_fh, @args)
 
 Takes a L<SVK::XD> object, the command name, the output scalar reference,
@@ -153,71 +157,81 @@ sub invoke {
     my ($pkg, $xd, $cmd, $output, @args) = @_;
     my ($help, $ofh, $ret);
     my $pool = SVN::Pool->new_default;
+
+    local *ARGV = [$cmd, @args];
     $ofh = select $output if $output;
+    $ret = eval {$pkg->dispatch ($xd ? (xd => $xd) : (),
+				 output => $output) };
+
+    $ofh = select STDERR unless $output;
     print $ret if $ret && $ret !~ /^\d+$/;
+    unless (ref($@)) {
+	print $@ if $@;
+    }
+    $ret = 1 if ($ret ? $ret !~ /^\d+$/ : $@);
+
+    undef $pool;
+    select $ofh if $ofh;
+    return ($ret || 0);
+}
+
+sub run_command {
+    my ($self, @args) = @_;
+    my $ret;
+
     local $SVN::Error::handler = sub {
 	my $error = $_[0];
 	my $error_message = $error->expanded_message();
 	$error->clear();
-	if ($cmd->handle_error ($error)) {
+	if ($self->handle_error ($error)) {
 	    die \'error handled';
         }
-	die $error_message."\n";
+
+        if ($ENV{SVKSVNBACKTRACE}) {
+            require Carp;
+            Carp::confess($error_message);
+        } else {
+            die $error_message."\n";
+        }
     };
 
-    local $@;
+    # XXX: this eval is too nasty
     eval {
-	$cmd = get_cmd ($pkg, $cmd, $xd);
-	$cmd->{svnconfig} = $xd->{svnconfig} if $xd;
-	$cmd->getopt (\@args, 'h|help|?' => \$help, 'encoding=s' => \$cmd->{encoding});
-	$cmd->_subcommand;
-
 	# Fake shell globbing on Win32 if we are called from main
 	if (IS_WIN32 and caller(1) eq 'main') {
 	    @args = map {
 		/[?*{}\[\]]/
 		    ? bsd_glob($_, File::Glob::GLOB_NOCHECK())
-		    : $_
-	    } @args;
+			: $_
+		    } @args;
 	}
-
-	# XXX: xd needs to know encoding too
-	$xd->{encoding} = $cmd->{encoding}
-	    if $xd;
-	if ($help || !(@args = $cmd->parse_arg(@args))) {
-	    select STDERR unless $output;
-	    $cmd->usage;
+	# XXX: xd needs to know encoding and ignore too
+	$self->{xd}{encoding} = $self->{encoding}
+	    if $self->{xd};
+	$self->{xd}{ignore} = $self->{ignore}
+	    if $self->{ignore};
+	if ($self->{help} || !(@args = $self->parse_arg(@args))) {
+	    select STDERR unless $self->{output};
+	    $self->usage;
 	}
 	else {
-	    $cmd->msg_handler ($SVN::Error::FS_NO_SUCH_REVISION);
-	    eval { $cmd->lock (@args);
-		   $xd->giant_unlock if $xd && !$cmd->{hold_giant};
-		   $ret = $cmd->run (@args) };
-	    $xd->unlock if $xd;
+	    $self->msg_handler ($SVN::Error::FS_NO_SUCH_REVISION);
+	    eval { $self->lock (@args);
+		   $self->{xd}->store if $self->{xd} && !$self->{hold_giant};
+		   $ret = $self->run (@args) };
+	    $self->{xd}->unlock if $self->{xd};
 	    die $@ if $@;
 	}
+
     };
+    # in case parse_arg dies, unlock giant
+    $self->{xd}->giant_unlock if $self->{xd} && ref ($self) && !$self->{hold_giant};
+    die $@ if $@;
+    return $ret;
+}
 
-    # in case parse_arg dies
-    $xd->giant_unlock if $xd && ref ($cmd) && !$cmd->{hold_giant};
-
-    $ofh = select STDERR unless $output;
-    print $ret if $ret && $ret !~ /^\d+$/;
-    # if an error handler terminates editor call, there will be stack trace
-    unless (ref($@)) {
-	if ($SVN::Core::VERSION gt '1.2.2') {
-	    print $@ if $@;
-	}
-	else {
-	    # if an error handler terminates editor call, there will be stack trace
-	    print $@ if $@ && $@ !~ m/\n.+\n.+\n/
-	}
-    }
-    $ret = 1 if ($ret ? $ret !~ /^\d+$/ : $@);
-    undef $cmd; undef $pool; # this needs to happen before finish select
-    select $ofh if $ofh;
-
-    return ($ret || 0);
+sub error_cmd {
+    loc ("Command not recognized, try %1 help.\n", $0);
 }
 
 =head3 getopt ($argv, %opt)
@@ -242,6 +256,16 @@ sub getopt {
 	if defined $recursive;
 }
 
+sub command_options {
+    my $self = shift;
+    $self->{recursive} = $self->opt_recursive;
+    my %opt;
+    $opt{$self->{recursive} ? 'N|non-recursive' : 'R|recursive'} =
+	sub { $self->{recursive} = !$self->{recursive} }
+	    if defined $self->{recursive};
+    ($self->options, %opt);
+}
+
 =head2 Instance Methods
 
 C<SVK::Command-E<gt>invoke> loads the corresponding class
@@ -264,13 +288,6 @@ value will be stored in C<recursive>.
 
 =cut
 
-sub options { () }
-
-sub _opt_map {
-    my ($self, %opt) = @_;
-    return map {$_ => \$self->{$opt{$_}}} sort keys %opt;
-}
-
 =head3 parse_arg (@args)
 
 This method is called with the remaining arguments after parsing named
@@ -310,7 +327,7 @@ sub run {
 =head2 Utility Methods
 
 Except for C<arg_depotname>, all C<arg_*> methods below returns a
-L<SVK::Target> object, which consists of a hash with the following keys:
+L<SVK::Path> object, which consists of a hash with the following keys:
 
 =over
 
@@ -341,41 +358,22 @@ Argument is a number of checkout paths.
 =cut
 
 sub arg_condensed {
-    my ($self, @arg) = @_;
-    return if $#arg < 0;
-
-    s{[/\Q$SEP\E]$}{}o for @arg; # XXX band-aid
-
-    my ($report, $copath, @targets )= $self->{xd}->condense (@arg);
-
+    my $self = shift;
+    my @args = map { $self->arg_copath($_) } @_;
     if ($self->{recursive}) {
 	# remove redundant targets when doing recurisve
 	# if have '' in targets then it means everything
-	my @newtarget = @targets;
-	for my $anchor (sort {length $a <=> length $b} @targets) {
-	    @newtarget = grep { length $anchor ? $_ eq $anchor || index ($_, "$anchor/") != 0
-				               : 0} @newtarget;
+	my @newtarget = @args;
+	for my $anchor (sort {length $a->copath <=> length $b->copath} @args) {
+	    local $SIG{__WARN__} = sub {};# path::class bug on /foo/bar vs /foo
+	    @newtarget = grep {
+		$anchor->copath eq $_->copath ||
+		!Path::Class::dir($anchor->copath)->subsumes($_->copath) } @newtarget;
 	}
-	@targets = @newtarget;
+	@args = @newtarget;
     }
 
-    my ($repospath, $path, undef, $cinfo, $repos) = $self->{xd}->find_repos_from_co ($copath, 1);
-    my $target = SVK::Target->new
-	( repos => $repos,
-	  repospath => $repospath,
-	  depotpath => $cinfo->{depotpath},
-	  copath => $copath,
-	  path => $path,
-	  report => $report,
-	  targets => @targets ? \@targets : undef );
-    my $root = $target->root ($self->{xd});
-    until ($root->check_path ($target->{path}) == $SVN::Node::dir) {
-	my $targets = delete $target->{targets};
-	$target->anchorify;
-	$target->{targets} = [map {"$target->{targets}[0]/$_"} @$targets]
-	    if $targets;
-    }
-    return $target;
+    return $self->{xd}->target_condensed(@args);
 }
 
 =head3 arg_uri_maybe ($arg, $no_new_mirror)
@@ -390,28 +388,22 @@ sub arg_uri_maybe {
     my ($self, $arg, $no_new_mirror) = @_;
 
     is_uri($arg) or return $self->arg_depotpath($arg);
-    HAS_SVN_MIRROR or die loc("cannot load SVN::Mirror");
 
     $arg =~ s{/?$}{/}; # add a trailing slash at the end
 
     require URI;
     my $uri = URI->new($arg)->canonical or die loc("%1 is not a valid URI.\n", $arg);
     my $map = $self->{xd}{depotmap};
-    foreach my $depot (sort keys %$map) {
-        my $repos = eval { ($self->{xd}->find_repos ("/$depot/", 1))[2] } or next;
-	foreach my $path ( SVN::Mirror::list_mirror ($repos) ) {
-	    my $m = eval {SVN::Mirror->new (
-                repos => $repos,
-                get_source => 1,
-                target_path => $path,
-            ) } or next;
-
-            my $rel_uri = $uri->rel(URI->new("$m->{source}/")->canonical) or next;
+    foreach my $depotname (sort keys %$map) {
+        my $depot = eval { $self->{xd}->find_depot($depotname) } or next;
+	foreach my $path ($depot->mirror->entries) {
+	    my $m = $depot->mirror->get($path);
+            my $rel_uri = $uri->rel(URI->new($m->url."/")->canonical) or next;
             next if $rel_uri->eq($uri);
             next if $rel_uri =~ /^\.\./;
 
-            my $depotpath = catdepot($depot, $path, $rel_uri);
-            $depotpath = "/$depotpath" if !length($depot);
+            my $depotpath = catdepot($depot->depotname, $path, $rel_uri);
+            $depotpath = "/$depotpath" if !length($depot->depotname);
             return $self->arg_depotpath($depotpath);
 	}
     }
@@ -419,7 +411,10 @@ sub arg_uri_maybe {
     die loc ("URI not allowed here: %1.\n", $no_new_mirror)
 	if $no_new_mirror;
 
-    print loc("New URI encountered: %1\n", $uri);
+    # this is going to take a while, release giant lock
+    $self->{xd}->giant_unlock;
+
+    $logger->info(loc("New URI encountered: %1\n", $uri));
 
     my $depots = join('|', map quotemeta, sort keys %$map);
     my ($base_uri, $rel_uri);
@@ -481,10 +476,9 @@ usually good enough.
     # If we're mirroring via svn::mirror, not mirroring the whole history
     # is an option
     my ($m, $answer);
-    ($m,undef) = SVN::Mirror::is_mirrored ($target->{'repos'}, 
-                                           $target->{'path'}) if (HAS_SVN_MIRROR);
-    # If the user is mirroring from svn                                       
-    if (UNIVERSAL::isa($m,'SVN::Mirror::Ra'))  {                                
+    $m = $target->is_mirrored;
+    # If the user is mirroring from svn
+    if ($m) {
         print loc("
 svk needs to mirror the remote repository so you can work locally.
 If you're mirroring a single branch, it's safe to use any of the options
@@ -512,6 +506,8 @@ break history-sensitive merging within the mirrored path.
         $answer = 'a';
     }
 
+    eval {
+
     $self->command(
         sync => {
             skip_to => (
@@ -523,7 +519,16 @@ break history-sensitive merging within the mirrored path.
         }
     )->run ($target);
 
-    my $depotpath = length ($rel_uri) ? "$target->{depotpath}/$rel_uri" : $target->{depotpath};
+    $self->{xd}->giant_lock;
+
+    };
+
+    my $depotpath = length ($rel_uri) ? $target->depotpath."/$rel_uri" : $target->depotpath;
+    if (my $err = $@) {
+	print loc("Unable to complete initial sync: %1", $err);
+	die loc("Run svk sync %1, and run the %2 command again.\n", $depotpath, lc((ref($self) =~ m/::([^:]*)$/)[0]));
+    }
+
     return $self->arg_depotpath($depotpath);
 }
 
@@ -537,21 +542,29 @@ handles it via C<arg_uri_maybe>.
 sub arg_co_maybe {
     my ($self, $arg, $no_new_mirror) = @_;
 
-    $arg = $self->arg_uri_maybe($arg, $no_new_mirror)->{depotpath}
+    $arg = $self->arg_uri_maybe($arg, $no_new_mirror)->depotpath
 	if is_uri($arg);
 
     my $rev = $arg =~ s/\@(\d+)$// ? $1 : undef;
     my ($repospath, $path, $copath, $cinfo, $repos) =
 	$self->{xd}->find_repos_from_co_maybe ($arg, 1);
     from_native ($path, 'path', $self->{encoding});
-    return SVK::Target->new
+    my ($view, $revision, $subpath);
+    if (($view, $revision, $subpath) = $path =~ m{^/\^([\w/\-_]+)(?:\@(\d+)(.*))?$}) {
+	$revision ||= $repos->fs->youngest_rev;
+	($path, $view) = SVK::Command->create_view ($repos, $view, $revision, $subpath);
+    }
+
+    $rev ||= $cinfo->{revision} if defined $copath;
+    return $self->{xd}->create_path_object
 	( repos => $repos,
 	  repospath => $repospath,
 	  depotpath => $cinfo->{depotpath} || $arg,
-	  copath => $copath,
-	  report => $copath ? File::Spec->canonpath ($arg) : $arg,
 	  path => $path,
+	  view => $view,
 	  revision => $rev,
+	  copath_anchor => $copath,
+	  report => File::Spec->canonpath($arg),
 	);
 }
 
@@ -564,22 +577,30 @@ Argument is a checkout path.
 sub arg_copath {
     my ($self, $arg) = @_;
     my ($repospath, $path, $copath, $cinfo, $repos) = $self->{xd}->find_repos_from_co ($arg, 1);
+    my ($root);
+    my ($view, $rev, $subpath);
 
     if ($copath =~ m/([\x00-\x19\x7f])/) { # XXX: why isn't \c[ working?
 	die loc("Invalid control character '%1' in path '%2'\n",
 		sprintf("0x%02X", ord($1)), $arg);
     }
 
+    if (($view, $rev, $subpath) = $path =~ m{^/\^([\w/\-_]+)(?:\@(\d+)(.*))?$}) {
+	($path, $view) = $self->create_view ($repos, $view, $rev, $subpath);
+    }
+
     from_native ($path, 'path', $self->{encoding});
-    return SVK::Target->new
-	( repos => $repos,
-	  repospath => $repospath,
-	  report => File::Spec->canonpath ($arg),
-	  copath => $copath,
-	  path => $path,
-	  cinfo => $cinfo,
-	  depotpath => $cinfo->{depotpath},
-	);
+    return SVK::Path::Checkout->real_new
+	({ report => File::Spec->canonpath ($arg),
+	   copath_anchor => $copath,
+	   xd => $self->{xd},
+	   source => $self->{xd}->create_path_object
+	   ( repos => $repos,
+	     repospath => $repospath,
+	     path => $path,
+	     view => $view,
+	     revision => $cinfo->{revision}, # make this sane!
+	     depotpath => $cinfo->{depotpath} ) });
 }
 
 =head3 arg_depotpath ($arg)
@@ -588,22 +609,83 @@ Argument is a depotpath, including the slashes and depot name.
 
 =cut
 
+sub _resolve_anchor {
+    my ($repos, $base, $anchor) = @_;
+    # XXX
+    $anchor = Path::Class::Dir->new_foreign('Unix', $anchor);
+    $anchor =~ s/^\&\:// or return $anchor;
+    $anchor = Path::Class::Dir->new_foreign('Unix', $anchor);
+    my ($uuid, $path) = find_svm_source($repos, "$base");
+    return $anchor->relative($path)->absolute($base);
+}
+
+sub create_view {
+    my ($self, $repos, $view, $rev, $subpath) = @_;
+    my $fs = $repos->fs;
+    my $viewspec = Path::Class::File->new_foreign('Unix', '/', "$view");
+    my ($viewbase, $viewname) = ($viewspec->parent, $viewspec->basename);
+    $rev = $fs->youngest_rev unless defined $rev;
+    require SVK::View;
+    my $viewobj = SVK::View->new
+	({ name => $viewname, base => $viewbase,
+	   revision => $rev, pool => SVN::Pool->new });
+    $viewobj->pool(SVN::Pool->new);
+    my $root = $fs->revision_root($rev);
+    my $content = $root->node_prop("$viewbase", "svk:view:$viewname");
+    die loc("Unable to create view '%1' from on %2 for revision %3.\n",
+	    $viewname, $viewbase, $rev)
+	unless defined $content;
+    my ($anchor, @content) = grep { $_ && !m/^#/ } $content =~ m/^.*$/mg;
+    $anchor = _resolve_anchor($repos, $viewbase, $anchor);
+    die loc("Unable to create view '%1' from on %2 for revision %3.\n",
+	    $viewname, $viewbase, $rev)
+	unless $root->check_path("$anchor");
+    $viewobj->anchor($anchor);
+
+    $root->dir_entries("$anchor"); # XXX: for some reasons fsfs needs refresh
+
+    for (@content) {
+	my ($del, $path, $target) = m/\s*(-)?(\S+)\s*(\S+)?\s*$/ or die "can't parse $_";
+	my $abspath = Path::Class::Dir->new_foreign('Unix', $path)
+	    ->absolute($anchor);
+	if (defined $target) {
+	    $target = Path::Class::Dir->new_foreign('Unix', $target);
+	    $target = $target->absolute($anchor)
+		unless $target->is_absolute;
+	}
+	if ($del) {
+	    warn "path not required" if defined $target;
+	    $viewobj->add_map($abspath, undef);
+	}
+	else {
+	    die "path required" unless defined $target;
+	    $viewobj->add_map($abspath, $target);
+	}
+    }
+
+    $subpath = '' unless defined $subpath;
+    return (length $subpath ? $anchor eq '/' ? $subpath : $anchor.$subpath
+	    : $anchor->stringify, $viewobj);
+}
+
 sub arg_depotpath {
     my ($self, $arg) = @_;
-
+    my $root;
     my $rev = $arg =~ s/\@(\d+)$// ? $1 : undef;
-    my ($repospath, $path, $repos) = $self->{xd}->find_repos ($arg, 1);
+    my ($depot, $path) = $self->{xd}->find_depotpath($arg);
+    my $view;
     from_native ($path, 'path', $self->{encoding});
-    my $target = SVK::Target->new
-	( repos => $repos,
-	  repospath => $repospath,
+    if (($view) = $path =~ m{^/\^([\w\-_/]+)$}) {
+	($path, $view) = $self->create_view($depot->repos, $view, $rev);
+    }
+
+    return $self->{xd}->create_path_object
+	( depot => $depot,
 	  path => $path,
 	  report => $arg,
 	  revision => $rev,
-	  depotpath => $arg,
+	  view => $view,
 	);
-    $target->{depotpath} = '/'.$target->depotname.$path;
-    return $target;
 }
 
 =head3 arg_depotroot ($arg)
@@ -619,9 +701,7 @@ sub arg_depotroot {
     local $@;
     $arg = eval { $self->arg_co_maybe ($arg || '')->new (path => '/') }
            || $self->arg_depotpath ("//");
-    $arg->as_depotpath;
-
-    return $arg;
+    return $arg->as_depotpath->refresh_revision;
 }
 
 =head3 arg_depotname ($arg)
@@ -649,6 +729,20 @@ sub arg_path {
     return abs_path ($arg);
 }
 
+=head3 apply_revision($target)
+
+Apply the given revision from command line to C<$target>.
+
+=cut
+
+sub apply_revision {
+    my ($self, $target) = @_;
+    $target = $target->source if $target->isa('SVK::Path::Checkout');
+    return $target unless defined $self->{rev};
+
+    return $target->seek_to( $self->resolve_revision($target, $self->{rev}) );
+}
+
 =head3 parse_revlist ()
 
 Parse -c or -r to a list of [from, to] pairs.
@@ -656,32 +750,17 @@ Parse -c or -r to a list of [from, to] pairs.
 =cut
 
 sub parse_revlist {
-    my $self = shift;
+    my ($self,$target) = @_;
     die loc("Revision required.\n") unless $self->{revspec} or $self->{chgspec};
     die loc("Can't assign --revision and --change at the same time.\n")
 	if $self->{revspec} and $self->{chgspec};
     my ($fromrev, $torev);
-    if ($self->{chgspec}) {
-	my @revlist;
-	for (split (',', $self->{chgspec})) {
-	    my $reverse;
-	    if (($fromrev, $torev) = m/^(\d+)-(\d+)$/) {
-		--$fromrev;
-	    }
-	    elsif (($reverse, $torev) = m/^(-?)(\d+)$/) {
-		$fromrev = $torev - 1;
-		($fromrev, $torev) = ($torev, $fromrev) if $reverse;
-	    }
-	    else {
-		die loc("Change spec %1 not recognized.\n", $_);
-	    }
-	    push @revlist , [$fromrev, $torev];
-	}
-	return @revlist;
-    }
+
+    my @revlist = $self->resolve_chgspec($target);
+    return @revlist if(@revlist);
 
     # revspec
-    if (($fromrev, $torev) = $self->{revspec} =~ m/^(\d+):(\d+)$/) {
+    if (($fromrev, $torev) = $self->resolve_revspec($target)) {
 	return ([$fromrev, $torev]);
     }
     else {
@@ -701,8 +780,8 @@ XXX Undocumented
 sub lock_target {
     my $self = shift;
     for my $target (@_) {
-	$self->{xd}->lock ($target->{copath})
-	    if $target->{copath};
+	$self->{xd}->lock ($target->copath_anchor)
+	    if $target->isa('SVK::Path::Checkout');
     }
 }
 
@@ -714,12 +793,12 @@ XXX Undocumented
 
 sub lock_coroot {
     my $self = shift;
-    my @tgt = map { $_->{copath} }
-	grep { defined $_->{copath} } @_;
+    my @tgt = map { $_->copath($_->{copath_target}) }
+	grep { $_->isa('SVK::Path::Checkout') } @_;
     return unless @tgt;
     my %roots;
     for (@tgt) {
-	my (undef, $coroot) = $self->{xd}{checkout}->get($_);
+	my (undef, $coroot) = $self->{xd}{checkout}->get($_, 1);
 	$roots{$coroot}++;
     }
     $self->{xd}->lock($_)
@@ -881,7 +960,7 @@ sub command {
 
     my $cmd = (
         $is_rebless ? bless($self, "SVK::Command::$command")
-                    : "SVK::Command::$command"->new ($self->{xd})
+                    : "SVK::Command::$command"->new (xd => $self->{xd})
     );
     $cmd->{$_} = $args->{$_} for sort keys %$args;
 
@@ -904,14 +983,14 @@ sub rebless {
 sub find_checkout_anchor {
     my ($self, $target, $track_merge, $track_sync) = @_;
 
-    my $entry = $self->{xd}{checkout}->get ($target->{copath});
+    my $entry = $self->{xd}{checkout}->get ($target->copath_anchor);
     my $anchor_target = $self->arg_depotpath ($entry->{depotpath});
 
     return ($anchor_target, undef) unless $track_merge;
 
     my @rel_path = split(
         '/',
-        abs2rel ($target->{path}, $anchor_target->{path}, undef, '/')
+        abs2rel ($target->path_anchor, $anchor_target->path_anchor, undef, '/')
     );
 
     my $copied_from;
@@ -968,17 +1047,101 @@ svk use the default.
 
 	my $target = $self->arg_depotpath ($path);
 	last if $allow_exist or $target->root->check_path ($target->path) == $SVN::Node::none;
-	print loc ("Path %1 already exists.\n", $path);
+	$logger->warn(loc ("Path %1 already exists.", $path));
     }
 
     return $path;
 }
 
+
+=head3 run_command_recursively($target, $code)
+
+Traverse C<$target> and and invoke C<$code> with each node.
+
+=cut
+
+sub _run_code {
+    my ($self, $target, $code, $level, $errs, $kind) = @_;
+    eval { $code->( $target, $kind, $level ) };
+    if ($@) {
+	print $@;
+	push @$errs, "$@";
+    }
+}
+
+sub run_command_recursively {
+    my ( $self, $target, $code, $errs, $newline, $level ) = @_;
+    my $root = $target->root;
+    my $kind = $root->check_path( $target->path_anchor );
+    $self->_run_code($target, $code, -1, $errs, $kind);
+    $self->_descend_with( $target, $code, $errs, 1 )
+        if $kind == $SVN::Node::dir
+        && $self->{recursive}
+        && ( !$self->{depth} || 0 < $self->{depth} );
+    print "\n" if $newline;
+}
+
+sub _descend_with {
+    my ($self, $target, $code, $errs, $level) = @_;
+    my $root = $target->root;
+    my $entries = $root->dir_entries ($target->path_anchor);
+    my $pool = SVN::Pool->new_default;
+    for (sort keys %$entries) {
+	$pool->clear;
+	my $kind = $entries->{$_}->kind;
+	next if $kind == $SVN::Node::unknown;
+	my $child = $target->new->descend($_);
+
+        $self->_run_code($child, $code, $level, $errs, $kind);
+	my $isdir = ($kind == $SVN::Node::dir);
+	if ($isdir && $self->{recursive} && (!$self->{'depth'} || ( $level  < $self->{'depth'}))) {
+	    $self->_descend_with($child, $code, $errs, $level+1);
+	}
+    }
+}
+
+
+## Resolve the correct revision numbers given by "-c"
+sub resolve_chgspec {
+    my ($self,$target) = @_;
+    my @revlist;
+    my ($fromrev,$torev);
+    if(my $chgspec = $self->{chgspec}) {
+	for (split (',', $self->{chgspec})) {
+	    my $reverse;
+	    if (($fromrev, $torev) = m/^(\d+)-(\d+)$/) {
+		--$fromrev;
+	    }
+	    elsif (($reverse, $torev) = m/^(-?)(\d+)$/) {
+		$fromrev = $torev - 1;
+		($fromrev, $torev) = ($torev, $fromrev) if $reverse;
+	    }
+	    else {
+		eval { $torev = $self->resolve_revision($target,$_); };
+		die loc("Change spec %1 not recognized.\n", $_) if($@);
+		if ($torev < 0) {
+		    $fromrev = -$torev;
+		    $torev = $fromrev - 1;
+		}
+		else {
+		    $fromrev = $torev - 1;
+		}
+	    }
+	    push @revlist , [$fromrev, $torev];
+	}
+    }
+    return @revlist;
+}
+
+# Looks in revspec or change (not in chgspec, which is more complicated)
+
 sub resolve_revspec {
     my ($self,$target) = @_;
-    my $fs = $target->{repos}->fs;
+    my $fs = $target->repos->fs;
     my $yrev = $fs->youngest_rev;
     my ($r1,$r2);
+    die loc("Can't assign --revision and --change at the same time.\n")
+      if defined $self->{revspec} and defined $self->{change};
     if (my $revspec = $self->{revspec}) {
         if ($#{$revspec} > 1) {
             die loc ("Invalid -r.\n");
@@ -988,6 +1151,19 @@ sub resolve_revspec {
                 $self->resolve_revision($target,$_);
             } @$revspec;
         }
+    } elsif (defined(my $change = $self->{change})) {
+        my $flip;
+        $flip = 1 if $change =~ s/^-//;
+
+        my $r = $self->resolve_revision($target, $change);
+
+        if ($r == 0) {
+            die loc("There is no change 0.\n");
+        } elsif ($flip) {
+            ($r1, $r2) = ($r, $r-1);
+        } else {
+            ($r1, $r2) = ($r-1, $r);
+        }
     }
     return($r1,$r2);
 }
@@ -995,22 +1171,25 @@ sub resolve_revspec {
 sub resolve_revision {
     my ($self,$target,$revstr) = @_;
     return unless defined $revstr;
-    my $fs = $target->{repos}->fs;
+    my $fs = $target->repos->fs;
     my $yrev = $fs->youngest_rev;
     my $rev;
-    if($revstr =~ /^HEAD$/) {
+    if($revstr =~ /^HEAD$/i) {
         $rev = $self->find_head_rev($target);
-    } elsif ($revstr =~ /^BASE$/) {
+    } elsif ($revstr =~ /^BASE$/i) {
         $rev = $self->find_base_rev($target);
     } elsif ($revstr =~ /\{(\d\d\d\d-\d\d-\d\d)\}/) { 
         my $date = $1; $date =~ s/-//g;
         $rev = $self->find_date_rev($target,$date);
-    } elsif (HAS_SVN_MIRROR && (my ($rrev) = $revstr =~ m'^(\d+)@$')) {
-	if (my ($m) = SVN::Mirror::is_mirrored ($target->{repos}, $target->{path})) {
-	    $rev = $m->find_local_rev ($rrev);
+    } elsif ((my ($minus, $rrev) = $revstr =~ m'^(-)?(\d+)@$')) {
+	if (my $m = $target->is_mirrored) {
+	    $rev = $m->find_local_rev($rrev);
 	}
 	die loc ("Can't find local revision for %1 on %2.\n", $rrev, $target->path)
 	    unless defined $rev;
+	$rev *= $minus ? -1 : 1;
+    } elsif ($revstr =~ /^-\d+$/) {
+        $rev = $self->find_head_rev($target) + $revstr;
     } elsif ($revstr =~ /\D/) {
         die loc("%1 is not a number.\n",$revstr)
     } else {
@@ -1022,10 +1201,10 @@ sub resolve_revision {
 sub find_date_rev {
     my ($self,$target,$date) = @_;
     # $date should be in yyyymmdd format
-    my $fs = $target->{repos}->fs;
+    my $fs = $target->repos->fs;
     my $yrev = $fs->youngest_rev;
 
-    my ($rev,$last);
+    my $rev = 0;
     traverse_history (
         root        => $fs->revision_root($yrev),
         path        => $target->path,
@@ -1034,29 +1213,28 @@ sub find_date_rev {
             my $revdate = $props->{'svn:date'};
             $revdate =~ s/T.*$//; $revdate =~ s/-//g;
             if($date > $revdate) {
-                $rev = ($last || $_[1]);
+                $rev = $_[1];
                 return 0;
             }
-            $last = $_[1];
             return 1;
         },
     );
-    return $rev || $last;
+    return $rev;
 }
 
 
 sub find_base_rev {
     my ($self,$target) = @_;
     die(loc("BASE can only be issued with a check-out path\n"))
-        unless(defined($target->{copath}));
+        unless $target->isa('SVK::Path::Checkout');
     my $rev = $self->{xd}{checkout}->get($target->copath)->{revision};
     return $rev;
 }
 
 sub find_head_rev {
     my ($self,$target) = @_;
-    $target->as_depotpath;
-    my $fs = $target->{repos}->fs;
+    $target = $target->as_depotpath;
+    my $fs = $target->repos->fs;
     my $yrev = $fs->youngest_rev;
     my $rev;
     traverse_history (
@@ -1079,17 +1257,3 @@ __DATA__
 
 L<SVK>, L<SVK::XD>, C<SVK::Command::*>
 
-=head1 AUTHORS
-
-Chia-liang Kao E<lt>clkao@clkao.orgE<gt>
-
-=head1 COPYRIGHT
-
-Copyright 2003-2005 by Chia-liang Kao E<lt>clkao@clkao.orgE<gt>.
-
-This program is free software; you can redistribute it and/or modify it
-under the same terms as Perl itself.
-
-See L<http://www.perl.com/perl/misc/Artistic.html>
-
-=cut
