@@ -77,7 +77,7 @@ sub _log_remote_rev {
     my ( $target, $remoteonly ) = @_;
 
     # don't bother if this repository has no mirror
-    return if !$target->mirror->_entries;
+    return unless $target->mirror->entries;
 
     # we might be running log on a path containing mirrors.
     # FIXME: resolve when log outside mirror anchor
