@@ -1,3 +1,53 @@
+# BEGIN BPS TAGGED BLOCK {{{
+# COPYRIGHT:
+# 
+# This software is Copyright (c) 2003-2006 Best Practical Solutions, LLC
+#                                          <clkao@bestpractical.com>
+# 
+# (Except where explicitly superseded by other copyright notices)
+# 
+# 
+# LICENSE:
+# 
+# 
+# This program is free software; you can redistribute it and/or
+# modify it under the terms of either:
+# 
+#   a) Version 2 of the GNU General Public License.  You should have
+#      received a copy of the GNU General Public License along with this
+#      program.  If not, write to the Free Software Foundation, Inc., 51
+#      Franklin Street, Fifth Floor, Boston, MA 02110-1301 or visit
+#      their web page on the internet at
+#      http://www.gnu.org/copyleft/gpl.html.
+# 
+#   b) Version 1 of Perl's "Artistic License".  You should have received
+#      a copy of the Artistic License with this package, in the file
+#      named "ARTISTIC".  The license is also available at
+#      http://opensource.org/licenses/artistic-license.php.
+# 
+# This work is distributed in the hope that it will be useful, but
+# WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+# General Public License for more details.
+# 
+# CONTRIBUTION SUBMISSION POLICY:
+# 
+# (The following paragraph is not intended to limit the rights granted
+# to you to modify and distribute this software under the terms of the
+# GNU General Public License and is only of importance to you if you
+# choose to contribute your changes and enhancements to the community
+# by submitting them to Best Practical Solutions, LLC.)
+# 
+# By intentionally submitting any modifications, corrections or
+# derivatives to this work, or any other work intended for use with SVK,
+# to Best Practical Solutions, LLC, you confirm that you are the
+# copyright holder for those contributions and you grant Best Practical
+# Solutions, LLC a nonexclusive, worldwide, irrevocable, royalty-free,
+# perpetual, license to use, copy, create derivative works based on
+# those contributions, and sublicense and distribute those contributions
+# and any derivatives thereof.
+# 
+# END BPS TAGGED BLOCK }}}
 package SVK::Command::Patch;
 use strict;
 use SVK::Version;  our $VERSION = $SVK::VERSION;
@@ -8,7 +58,6 @@ use SVK::Patch;
 use SVK::Merge;
 use SVK::Editor::Merge;
 use SVK::I18N;
-use SVK::Util qw (resolve_svm_source);
 use SVK::Command::Log;
 
 
@@ -27,7 +76,18 @@ sub options {
 }
 
 sub parse_arg {
-    # always return help
+    my ($self, @args) = @_;
+    my $cmd = shift @args or return;
+
+    # Try to find a subcommand for invocations such as "svk patch ls"
+    my %options = $self->options;
+    while (my ($names, $canonical) = each %options) {
+        $cmd =~ /\A$names\z/ or next;
+        bless($self, "SVK::Command::Patch::$canonical");
+        return($self->parse_arg(@args));
+    }
+
+    # Can't find a subcommand - return help
     return;
 }
 
@@ -77,7 +137,7 @@ use base qw/SVK::Command::Patch::FileRequired/;
 
 sub run {
     my ($self, $patch) = @_;
-    print YAML::Dump ($patch);
+    print YAML::Syck::Dump ($patch);
     return;
 }
 
@@ -85,14 +145,15 @@ package SVK::Command::Patch::test;
 
 use base qw/SVK::Command::Patch::FileRequired/;
 use SVK::I18N;
+use SVK::Logger;
 
 sub run {
     my ($self, $patch, $not_applicable) = @_;
 
     return $not_applicable if $not_applicable;
     if (my $conflicts = $patch->apply (1)) {
-	print loc("%*(%1,conflict) found.\n", $conflicts);
-	print loc("Please do a merge to resolve conflicts and regen the patch.\n");
+	$logger->error(loc("%*(%1,conflict) found.", $conflicts));
+	$logger->error(loc("Please do a merge to resolve conflicts and regen the patch."));
     }
 
     return;
@@ -149,7 +210,6 @@ use base qw/SVK::Command::Patch/;
 sub parse_arg { undef }
 sub run {
     my ($self) = @_;
-    my (undef, undef, $repos) = $self->{xd}->find_repos ('//', 1);
     opendir my $dir, $self->{xd}->patch_directory;
     foreach my $file (readdir ($dir)) {
 	next if $file =~ /^\./;
@@ -174,10 +234,11 @@ sub run {
     my $mergecmd = $self->command ('merge');
     $mergecmd->getopt (\@args);
     my $dst = $self->arg_co_maybe ($args[0] || '');
-    $self->lock_target ($dst) if $dst->{copath};
+    $self->lock_target ($dst) if $dst->isa('SVK::Path::Checkout');
     my $ticket;
-    $mergecmd->get_commit_message ($patch->{log}) unless $dst->{copath};
-    my $merge = SVK::Merge->new (%$mergecmd, dst => $dst, repos => $dst->{repos});
+    $mergecmd->get_commit_message ($patch->{log})
+	unless $dst->isa('SVK::Path::Checkout');
+    my $merge = SVK::Merge->new (%$mergecmd, dst => $dst, repos => $dst->repos);
     $ticket = sub { $merge->get_new_ticket (SVK::Merge::Info->new ($patch->{ticket})) }
 	if $patch->{ticket} && $dst->universal->same_resource ($patch->{target});
     $patch->apply_to ($dst, $mergecmd->get_editor ($dst),
@@ -289,17 +350,3 @@ To apply a patch to the repository that someone else has sent you, run:
 
     svk patch --apply - < contributed_feature.patch
 
-=head1 AUTHORS
-
-Chia-liang Kao E<lt>clkao@clkao.orgE<gt>
-
-=head1 COPYRIGHT
-
-Copyright 2003-2005 by Chia-liang Kao E<lt>clkao@clkao.orgE<gt>.
-
-This program is free software; you can redistribute it and/or modify it
-under the same terms as Perl itself.
-
-See L<http://www.perl.com/perl/misc/Artistic.html>
-
-=cut
