@@ -303,7 +303,7 @@ sub open_file {
     # modified but rm locally - tag for conflict?
     my ($basepath, $fromrev) = $self->_resolve_base($path);
     $basepath = $path unless defined $basepath;
-    if ($self->inspector->exist($basepath, $pool) == $SVN::Node::file) {
+    if (defined $pdir && $self->inspector->exist($basepath, $pool) == $SVN::Node::file) {
 	$self->{info}{$path}{baseinfo} = [$basepath, $fromrev]
 	    if defined $fromrev;
 	$self->{info}{$path}{open} = [$pdir, $rev];
@@ -404,14 +404,14 @@ sub _retrieve_base
 }
 
 sub apply_textdelta {
-    my ($self, $path, $checksum, $pool) = @_;
+    my ($self, $path, $checksum, $ppool) = @_;
     return unless $path;
 
     my $info = $self->{info}{$path};
     my ($basepath, $fromrev) = $info->{baseinfo} ? @{$info->{baseinfo}} : ($path);
     my $fh = $info->{fh} = {};
-    if (($pool = $info->{fpool}) &&
-	($fh->{local} = $self->inspector->localmod($basepath, $checksum || '', $pool))) {
+    my $pool = $info->{fpool};
+    if ($pool && ($fh->{local} = $self->inspector->localmod($basepath, $checksum || '', $pool))) {
 	# retrieve base
 	unless ($info->{addmerge}) {
 	    $fh->{base} = [$self->_retrieve_base($path, $pool)];
@@ -426,7 +426,7 @@ sub apply_textdelta {
     $self->ensure_open ($path);
 
     my $handle = $self->{storage}->apply_textdelta ($self->{storage_baton}{$path},
-						    $checksum, $pool);
+						    $checksum, $ppool);
 
     if ($self->{storage_has_unwritable} && !$handle) {
 	delete $self->{notify}{status}{$path};
@@ -593,7 +593,10 @@ sub add_directory {
     return undef unless defined $pdir;
     my $pool = pop @arg;
     my $touched = $self->{notify}->node_status($path);
-    undef $touched if $touched && $touched eq 'C';
+    # This comes from R (D+A) where the D has conflict
+    if ($touched && $touched eq 'C') {
+	return undef;
+    }
     # Don't bother calling cb_exist (which might be expensive if the parent is
     # already added.
     if (!$self->{added}{$pdir} && !$touched &&
