@@ -274,6 +274,9 @@ Returns if the svn client library has replay capability
 
 sub has_replay_api {
     my $self = shift;
+
+    return if $ENV{SVKNORAREPLAY};
+
     return unless _p_svn_ra_session_t->can('replay');
 
     # The Perl bindings shipped with 1.4.0 has broken replay support
@@ -301,12 +304,14 @@ sub has_replay {
             $self->_ra_finished($ra);
             return $self->_has_replay(1);
         }
+        die $@ unless $err;
     }
     $self->_ra_finished($ra);
     # FIXME: if we do ^c here $err would be empty. do something else.
     return $self->_has_replay(0)
       if $err->apr_err == $SVN::Error::RA_NOT_IMPLEMENTED      # ra_svn
-      || $err->apr_err == $SVN::Error::UNSUPPORTED_FEATURE;    # ra_dav
+      || $err->apr_err == $SVN::Error::UNSUPPORTED_FEATURE     # ra_dav
+      || $err->apr_err == $SVN::Error::RA_DAV_REQUEST_FAILED;  # ra_dav (googlecode)
     die $err->expanded_message;
 }
 
@@ -372,15 +377,38 @@ sub _initialize_auth {
 =cut
 
 sub find_rev_from_changeset {
-    my ($self, $changeset) = @_;
+    my ($self, $changeset, $seekback) = @_;
     my $t = $self->mirror->get_svkpath;
-    return $t->search_revision
+
+    no warnings 'uninitialized'; # $s_changeset below may be undef
+
+    my $r = $t->search_revision
 	( cmp => sub {
 	      my $rev = shift;
-	      my $search = $t->mclone(revision => $rev);
               my $s_changeset = scalar $self->find_changeset($rev);
               return $s_changeset <=> $changeset;
           } );
+
+    return defined $r ? $r : () if $r || !$seekback;
+
+    my $result;
+    $r = $t->search_revision
+	( cmp => sub {
+	      my $rev = shift;
+
+              my $s_changeset = scalar $self->find_changeset($rev);
+
+	      if ($s_changeset > $changeset) {
+		  my $prev = $t->mclone(revision => $rev)->prev;
+		  $result = $prev
+		      if scalar $self->find_changeset($prev->revision) < $changeset;
+	      }
+	      return $s_changeset <=> $changeset;
+          } );
+
+    return unless $result;
+
+    return $result->revision;
 }
 
 =item find_changeset( $local_rev )
@@ -502,7 +530,7 @@ sub _get_sync_editor {
             return $target->as_url(
                 1,
                 $self->mirror->path . $path,
-                $self->find_rev_from_changeset($rev)
+                $self->find_rev_from_changeset($rev, 1)
             );
         }
     );
