@@ -953,6 +953,11 @@ sub str2time {
 
 sub time2str {
     my ($format, $time) = @_;
+    if (IS_WIN32) {
+	require Date::Format;
+	goto \&Date::Format::time2str;
+    }
+
     require POSIX;
     return POSIX::strftime($format, localtime($time) );
 }
@@ -969,7 +974,7 @@ sub find_dotsvk {
     while ( $p && $p ne $prev && -r $p ) {
 	$prev = $p;
 	my $svk = $p->subdir('.svk');
-	return $svk if -e $svk;
+	return $svk if -e $svk && -e $svk->file('floating');
 	$p = $p->parent();
     }
 
